/***************************************************************************/
/*                                                                         */
/* (c) Copyright IBM Corp. 1999  All rights reserved.                      */
/*                                                                         */
/* This sample program is owned by International Business Machines         */
/* Corporation or one of its subsidiaries ("IBM") and is copyrighted       */
/* and licensed, not sold.                                                 */
/*                                                                         */
/* You may copy, modify, and distribute this sample program in any         */
/* form without payment to IBM, for any purpose including developing,      */
/* using, marketing or distributing programs that include or are           */
/* derivative works of the sample program.                                 */
/*                                                                         */
/* The sample program is provided to you on an "AS IS" basis, without      */
/* warranty of any kind.  IBM HEREBY  EXPRESSLY DISCLAIMS ALL WARRANTIES,  */
/* EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED   */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.     */
/* Some jurisdictions do not allow for the exclusion or limitation of      */
/* implied warranties, so the above limitations or exclusions may not      */
/* apply to you.  IBM shall not be liable for any damages you suffer as    */
/* a result of using, modifying or distributing the sample program or      */
/* its derivatives.                                                        */
/*                                                                         */
/* Each copy of any portion of this sample program or any derivative       */
/* work,  must include the above copyright notice and disclaimer of        */
/* warranty.                                                               */
/*                                                                         */
/***************************************************************************/

/*													
Program name : PUBSUB												
Author       : Brian Homewood
Description  : Sample program that starts the NT pub/sub broker automatically
by creating a two broker hierarchy(parent/child relationship)

*/

/************************************************************************************

Function :  This programs starts the broker on child queuemanager and
            on parent queue manager. The QueueManager names are passed as arguments
            to this program.

Before running this program the following are the requirements
a) QueueManagers in the relationship are started
b) Brokers in a relationship reuire their queue mangers to  commuicate both
ways between each other, hence create channels for communication.					
c) Ensure that listener fro each queue manager has been started
d) create a separate Batch File to start the QueuManagers and Listeners.
This program was tested with MSVC++ 6.0 compiler


NOTE 1 : If you have problem starting the child broker,its recommended that
         you allow a delay of a few seconds between starting related brokers
         for the first time as this allows the parent broker to be running
         before the child registers with it

************************************************************************************
															
PUBSUB has 2 parameters - the queue manager name (optional)
                        - the parent queue manager name (optional)
Note: if only one parameter is supplied it is assumed to be the queue manager name

************************************************************************************/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/******************************************************************************/
/* Defines                                                                    */
/* QueueManager & Parent QueueManager				              */
/******************************************************************************/
#define OK        0                              /* define OK as zero         */
#define NOT_OK    1                              /* define NOT_OK as one      */
#ifndef TRUE
   #define TRUE   1
#endif
#ifndef FALSE
   #define FALSE  0
#endif
#ifndef BOOL
   #define BOOL unsigned int
#endif
#define BROKER_MSG  "MQSeries message broker for queue manager "

/******************************************************************************/
/* Global variables                                                           */
/******************************************************************************/
BOOL QMNameSupplied=FALSE;                       /* QM name supplied          */
BOOL parentQMNameSupplied=FALSE;                 /* parent QM name supplied   */

/******************************************************************************/
/* Function prototypes                                                        */
/******************************************************************************/
int StartQM(char *QMName);
int StartBroker(char *QMName);
int StartParentBroker(char *QMName,char *ParentQMName);
int EndBroker(char *QMName);
int BrokerNotRunning(char *QMName);

/******************************************************************************/
/*                                                                            */
/* Function: Main                                                             */
/* ==============                                                             */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Input Parameters:                                                          */
/* -----------------                                                          */
/* int argc                       - number of arguments supplied              */
/* char *argv[]                   - program arguments                         */
/*                                                                            */
/* Returns:                                                                   */
/* --------                                                                   */
/* OK                             - all is well                               */
/*                                                                            */
/******************************************************************************/
int main(int argc, char *argv[])
{
   int rc=OK;                                    /* return code               */
   char QMName[50]="";                           /* default QM name           */
   char parentQMName[50]="";                     /* parent QM name            */

   /***************************************************************************/
   /* First check we have been given correct arguments                        */
   /***************************************************************************/
   if (argc < 1 || argc > 3)
   {
      printf("Input is: %s 'QMgr name' 'parent QMgr name'.\n"
             "Note: the queue manager name and parent queue manager name are optional\n",
              argv[0]);
      exit(99);
   } /* endif */

   if (argc == 2)
   {
      QMNameSupplied = TRUE;
      strcpy(QMName, argv[1]);
   }
   else if (argc == 3)
   {
      QMNameSupplied = TRUE;
      parentQMNameSupplied = TRUE;
      strcpy(QMName, argv[2]);
      strcpy(parentQMName, argv[1]);
   } /* endif */


   if(QMNameSupplied){
	 rc = StartBroker(QMName);    /* start the broker          */
	 if(rc != OK)
		  return rc;
   }

   printf("Broker on Queue Manager %s started",QMName);
   if(parentQMNameSupplied){
	rc = StartParentBroker(QMName,parentQMName);	/* start the broker  on Parent Queue Manager        */
			if(rc != OK){
			  printf("Ending the Broker on Queu Manager %s ",QMName);
			  EndBroker(QMName);
			  return rc;
			}
	}
   return rc;
}                                                /* ------ END OF MAIN ------ */

/******************************************************************************/
/*                                                                            */
/* Function: StartBroker                                                      */
/*                                                                            */
/* Description: Start the broker                                              */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Function:                                                                  */
/* ---------                                                                  */
/* Starts the required broker                                                 */
/*                                                                            */
/* Input Parameters:                                                          */
/* -----------------                                                          */
/* char *QMName                   - queue manager name                        */
/*                                                                            */
/* In/Out Parameters:                                                         */
/* ------------------                                                         */
/* None                                                                       */
/*                                                                            */
/* Output Parameters:                                                         */
/* ------------------                                                         */
/* None                                                                       */
/*                                                                            */
/* Returns:                                                                   */
/* --------                                                                   */
/* OK                             - all is well                               */
/*                                                                            */
/******************************************************************************/
int StartBroker(char *QMName)
{
   int rc=OK;                                    /* return code               */
   char systemStr[128]="";                       /* system string             */

   strcpy(systemStr, "strmqbrk");
   strcat(systemStr, " -m ");
   strcat(systemStr, QMName);
   
   printf("Issuing:'%s'\n", systemStr);
   rc = system(systemStr);                       /* start the broker          */

   if (rc != OK)
      printf("Error strmqbrk return code=%d\n", rc);

   return rc;
}

/******************************************************************************/
/*                                                                            */
/* Function: StartParentBroker                                                */
/*                                                                            */
/* Description: Start the broker                                              */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Function:                                                                  */
/* ---------                                                                  */
/* Starts the required broker                                                 */
/*                                                                            */
/* Input Parameters:                                                          */
/* -----------------                                                          */
/* char *QMName                   - Queue manager name			      */
/* char *ParentQMName                   - Parent queue manager name           */
/*                                                                            */
/* In/Out Parameters:                                                         */
/* ------------------                                                         */
/* None                                                                       */
/*                                                                            */
/* Output Parameters:                                                         */
/* ------------------                                                         */
/* None                                                                       */
/*                                                                            */
/* Returns:                                                                   */
/* --------                                                                   */
/* OK                             - all is well                               */
/*                                                                            */
/******************************************************************************/
int StartParentBroker(char *QMName,char *ParentQMName)
{
   int rc=OK;                                    /* return code               */
   char systemStr[128]="";                       /* system string             */

   strcpy(systemStr, "strmqbrk");
   strcat(systemStr, " -m ");
   strcat(systemStr, ParentQMName);
   
   strcat(systemStr, " -p ");
   strcat(systemStr, QMName);

   printf("Issuing:'%s'\n", systemStr);
   rc = system(systemStr);                       /* start the broker          */

   if (rc != OK)
      printf("Error strmqbrk return code=%d\n", rc);

   return rc;
}

/******************************************************************************/
/*                                                                            */
/* Function: StartBroker                                                      */
/*                                                                            */
/* Description: Start the broker                                              */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Function:                                                                  */
/* ---------                                                                  */
/* Ends the required broker                                                   */
/*                                                                            */
/* Input Parameters:                                                          */
/* -----------------                                                          */
/* char *QMName                   - queue manager name                        */
/*                                                                            */
/* In/Out Parameters:                                                         */
/* ------------------                                                         */
/* None                                                                       */
/*                                                                            */
/* Output Parameters:                                                         */
/* ------------------                                                         */
/* None                                                                       */
/*                                                                            */
/* Returns:                                                                   */
/* --------                                                                   */
/* OK                             - all is well                               */
/*                                                                            */
/******************************************************************************/
int EndBroker(char *QMName)
{
   int rc=OK;                                    /* return code               */
   char systemStr[128]="";                       /* system string             */

   strcpy(systemStr, "endmqbrk");
   strcat(systemStr, " -m ");
   strcat(systemStr, QMName);
   
   printf("Issuing:'%s'\n", systemStr);
   rc = system(systemStr);                       /* start the broker          */

   if (rc != OK)
      printf("Error strmqbrk return code=%d\n", rc);

   return rc;
}

/******************************************************************************/
/*                                                                            */
/* Function: BrokerNotRunning                                                 */
/*                                                                            */
/* Description: Tests that the broker is not running                          */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Function:                                                                  */
/* ---------                                                                  */
/* Tests that the required broker is not running, by issuing the dspmqbrk     */
/* command and examining the output message.                                  */
/*                                                                            */
/* Input Parameters:                                                          */
/* -----------------                                                          */
/* char *QMName                   - queue manager name                        */
/*                                                                            */
/* In/Out Parameters:                                                         */
/* ------------------                                                         */
/* None                                                                       */
/*                                                                            */
/* Output Parameters:                                                         */
/* ------------------                                                         */
/* None                                                                       */
/*                                                                            */
/* Returns:                                                                   */
/* --------                                                                   */
/* OK                             - broker is not running                     */
/* NOT_OK                         - broker is already running or an error     */
/*                                  occured                                   */
/*                                                                            */
/******************************************************************************/
int BrokerNotRunning(char *QMName)
{
   int rc=OK;                                    /* return code               */
   FILE *pFile;                                  /* ptr to file               */
   char msgBuf[256];                             /* message buffer            */
   char textBuf[256];                            /* text buffer               */
   char systemStr[128]="";                       /* system string             */
   char savedChar;                               /* saved char                */
   int msgBufLen;                                /* message buffer length     */

   strcpy(systemStr, "dspmqbrk");

   if (QMNameSupplied)
   {
      strcat(systemStr, " -m ");
      strcat(systemStr, QMName);
   } /* endif */

   strcat(systemStr, " 2> stpubsub.tmp");

   printf("Issuing:'%s'\n", systemStr);
   rc = system(systemStr);                       /* display the broker status */

   /***************************************************************************/
   /* If the broker is not running, command dspmqbrk should display the       */
   /* message "MQSeries message broker for queue manager XXX not active."     */
   /***************************************************************************/
   if (rc == OK)
   {
      pFile = fopen("stpubsub.tmp", "r");        /* open text file for read   */

      if (pFile != NULL)
      {
         fgets(textBuf, 256, pFile);             /* get line from temp file   */
         fclose(pFile);                          /* close the file            */
      }
      else
      {
         printf("Could not open stpubsub.tmp\n");
         rc = NOT_OK;
      } /* endif */
   }
   else
   {
      printf("Error dspmqbrk return code=%d\n", rc);
   } /* endif */

   if (rc == OK)
   {
      strcpy(msgBuf, BROKER_MSG);
      strcat(msgBuf, QMName);
      strcat(msgBuf, "not active.");
      msgBufLen = strlen(msgBuf);
      savedChar = textBuf[msgBufLen];
      textBuf[msgBufLen] = '\0';                 /* remove surplus space char */

      if (strcmp(msgBuf, textBuf) != 0)
      {
         rc = NOT_OK;
         textBuf[msgBufLen] = savedChar;         /* restore message           */
         printf(textBuf);                        /* print original message    */
      } /* endif */
   } /* endif */

   remove("stpubsub.tmp");
   return rc;
}

